<?php
    $jobStartTime = time();

    // +---------------------------------------+
    // | define constants                      |
    // +---------------------------------------+

    define('phpAds_path', dirname(__FILE__));
    define('MYSQL_SCRIPT_DIR', '/home/mysql/scripts');
    define('DEBUG', true);

    //  db names
    define('phpAds_adminDb', 1);
    define('phpAds_rawDb', 2);

    //  usertypes
    define('phpAds_userDeliveryEngine', 1);
    define('phpAds_userMaintenance', 2);
    define('phpAds_userAdministrator', 3);
    define('phpAds_userAdvertiser', 4);
    define('phpAds_userPublisher', 5);

    //  actions
    define('phpAds_actionAdvertiserReportMailed', 0);
    define('phpAds_actionPublisherReportMailed', 1);
    define('phpAds_actionWarningMailed', 2);
    define('phpAds_actionDeactivationMailed', 3);
    define('phpAds_actionPriorityCalculation', 10);
    define('phpAds_actionPriorityAutoTargeting', 11);
    define('phpAds_actionDeactiveCampaign', 20);
    define('phpAds_actionActiveCampaign', 21);
    define('phpAds_actionAutoClean', 30);
    define('phpAds_actionBatchStatistics', 40);
    define('phpAds_installed', true);

    // +---------------------------------------+
    // | configure PHP                         |
    // +---------------------------------------+

    if (DEBUG) {
        ini_set('error_log', phpAds_path . '/log.txt');
        error_log('     '); error_log('================================');error_log('     ');
    }
    // Set time limit and ignore user abort
    if (!ini_get('safe_mode')) {
        @set_time_limit(300);
        @ignore_user_abort(true);
    }
    // Disable magic_quotes_runtime
    set_magic_quotes_runtime(0);

    // +---------------------------------------+
    // | load config and includes              |
    // +---------------------------------------+

    //  load config file
    $conf = parse_ini_file(phpAds_path . '/default.conf.ini', true);

    require_once phpAds_path . '/maintenance-common.php';

    debug('initial libs included correctly');

    // +---------------------------------------+
    // | db connect                            |
    // +---------------------------------------+

    phpAds_dbConnect();
    phpAds_LoadDbConfig();

    debug('db connection established, db config loaded');



	
	phpAds_countConversions('20040427160000', '20040427170000', '2004-04-27', '16');





function phpAds_countConversions($begin_timestamp, $end_timestamp, $day, $hour)
{
    global $conf, $report;
    debug('- running the 3rd job: phpAds_countConversions()');
    $time = time();

	$conf['tbl_adstats'] = 'stats_temp';

    $report .= "\tCounting the verbose conversions between ".$begin_timestamp." and ".$end_timestamp."...\n";
    
    // Create temporary heap table
    $sql = 'CREATE TEMPORARY TABLE `tmp_conversions` (
              `conversionlogid` mediumint(9) NOT NULL auto_increment,
              `conversionid` bigint(20) unsigned NOT NULL default "0",
              `campaignid` mediumint(9) NOT NULL default "0",
              `trackerid` mediumint(9) NOT NULL default "0",
              `userid` varchar(32) NOT NULL default "",
              `t_stamp` timestamp(14) NOT NULL,
              `host` varchar(255) NOT NULL default "",
              `country` char(2) NOT NULL default "",
              `cnv_logstats` enum("y","n") default "n",
              `cnv_clickwindow` mediumint(9) NOT NULL default "0",
              `cnv_viewwindow` mediumint(9) NOT NULL default "0",
              `action` enum("view","click") default NULL,
              `action_bannerid` mediumint(9) NOT NULL default "0",
              `action_zoneid` mediumint(9) NOT NULL default "0",
              `action_t_stamp` timestamp(14) NOT NULL,
              `action_host` varchar(255) NOT NULL default "",
              `action_source` varchar(50) NOT NULL default "",
              `action_country` char(2) NOT NULL default "",
              PRIMARY KEY  (`conversionlogid`)
            )';
    debug('- phpAds_countConversions() - Create temporary table: Started');
    $res = phpAds_dbQuery($sql)
        or $_report .= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Create temporary table: Ended');

    // Insert click conversions into temporary table
    $sql = 'INSERT INTO
                tmp_conversions
                (conversionid,
                campaignid,
                trackerid,
                userid,
                t_stamp,
                host,
                country,
                cnv_logstats,
                cnv_clickwindow,
                cnv_viewwindow,
                action,
                action_bannerid,
                action_zoneid,
                action_t_stamp,
                action_host,    
                action_source,
                action_country)
            SELECT
                cnv.conversionid AS conversionid
                ,b.campaignid AS campaignid
                ,cnv.trackerid AS trackerid
                ,cnv.userid AS userid
                ,cnv.t_stamp AS t_stamp
                ,cnv.host AS host
                ,cnv.country AS country
                ,t.logstats AS cnv_logstats
                ,t.clickwindow AS cnv_clickwindow
                ,t.viewwindow AS cnv_viewwindow
                ,"click" AS action
                ,c.bannerid AS action_bannerid
                ,c.zoneid AS action_zoneid
                ,c.t_stamp AS action_t_stamp
                ,c.host AS action_host
                ,c.source AS action_source
                ,c.country AS action_country
            FROM
                 '.$conf['tbl_adconversions'].' AS cnv
                ,'.$conf['tbl_adclicks'].' AS c
                ,'.$conf['tbl_campaigns_trackers'].' AS t
                ,'.$conf['tbl_banners'].' AS b
            WHERE
                t.trackerid=cnv.trackerid
                AND b.bannerid=c.bannerid
                AND b.campaignid=t.campaignid
                AND cnv.userid=c.userid
                AND cnv.userid != ""
                AND cnv.t_stamp >= '.$begin_timestamp.'
                AND cnv.t_stamp < '.$end_timestamp.'
                AND cnv.t_stamp < DATE_ADD(c.t_stamp, INTERVAL t.clickwindow SECOND)
                AND cnv.t_stamp >= c.t_stamp
                AND t.clickwindow > 0
            ORDER BY
                c.t_stamp DESC';
    debug('- phpAds_countConversions() - Insertion of click conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $_report .= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Insertion of click conversions: Ended');

    // Insert view conversions into the temporary table, obtaining and then dropping locks
    // on the appropriate tables.
    //
    // BEWARE!!!
    //
    // The tables names specified with the "AS" statement in the lock MUST be the same as
    // the "AS" table names specified in the select!
    $sql = 'LOCK TABLES
                 '.$conf['tbl_adconversions'].' AS cnv READ
                ,'.$conf['tbl_adviews'].' AS v READ
                ,'.$conf['tbl_campaigns_trackers'].' AS t READ
                ,'.$conf['tbl_banners'].' AS b READ
                ,tmp_conversions WRITE';
    debug('- phpAds_countConversions() - Locking tables for insertion of view conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Locking tables for insertion of view conversions: Ended');
    
    $sql = 'FLUSH TABLE
                 '.$conf['tbl_adconversions'].'
                ,'.$conf['tbl_adviews'].'
                ,'.$conf['tbl_campaigns_trackers'].'
                ,'.$conf['tbl_banners'].'
                ,tmp_conversions';
    debug('- phpAds_countConversions() - Flushing tables for insertion of view conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Flushing tables for insertion of view conversions: Ended');

    $sql = 'INSERT INTO 
                tmp_conversions
                (conversionid,
                campaignid,
                trackerid,
                userid,
                t_stamp,
                host,
                country,
                cnv_logstats,
                cnv_clickwindow,
                cnv_viewwindow,
                action,
                action_bannerid,
                action_zoneid,
                action_t_stamp,
                action_host,    
                action_source,
                action_country)
            SELECT
                cnv.conversionid AS conversionid
                ,b.campaignid AS campaignid
                ,cnv.trackerid AS trackerid
                ,cnv.userid AS userid
                ,cnv.t_stamp AS t_stamp
                ,cnv.host AS host
                ,cnv.country AS country
                ,t.logstats AS cnv_logstats
                ,t.clickwindow AS cnv_clickwindow
                ,t.viewwindow AS cnv_viewwindow
                ,"view" AS action
                ,v.bannerid AS action_bannerid
                ,v.zoneid AS action_zoneid
                ,v.t_stamp AS action_t_stamp
                ,v.host AS action_host
                ,v.source AS action_source
                ,v.country AS action_country
            FROM
                 '.$conf['tbl_adconversions'].' AS cnv
                ,'.$conf['tbl_adviews'].' AS v
                ,'.$conf['tbl_campaigns_trackers'].' AS t
                ,'.$conf['tbl_banners'].' AS b
            WHERE
                t.trackerid=cnv.trackerid
                AND b.bannerid=v.bannerid
                AND b.campaignid=t.campaignid
                AND cnv.userid=v.userid
                AND cnv.userid != ""
                AND cnv.t_stamp >= '.$begin_timestamp.'
                AND cnv.t_stamp < '.$end_timestamp.'
                AND cnv.t_stamp < DATE_ADD(v.t_stamp, INTERVAL t.viewwindow SECOND)
                AND cnv.t_stamp >= v.t_stamp
                AND t.viewwindow > 0
            ORDER BY
                v.t_stamp DESC';
    debug('- phpAds_countConversions() - Insertion of view conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Insertion of view conversions: Ended');

    $sql = 'UNLOCK TABLES';
    debug('- phpAds_countConversions() - Unlocking tables after insertion of view conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Unlocking tables after insertion of view conversions: Ended');

    $sql = 'INSERT INTO
                    '.$conf['tbl_conversionlog'].'
                    (conversionid,
                    campaignid,
                    trackerid,
                    userid,
                    t_stamp,
                    host,
                    country,
                    cnv_logstats,
                    cnv_clickwindow,
                    cnv_viewwindow,
                    action,
                    action_bannerid,
                    action_zoneid,
                    action_t_stamp,
                    action_host,    
                    action_source,
                    action_country)
                SELECT
                    conversionid,
                    campaignid,
                    trackerid,
                    userid,
                    t_stamp,
                    host,
                    country,
                    cnv_logstats,
                    cnv_clickwindow,
                    cnv_viewwindow,
                    action,
                    action_bannerid,
                    action_zoneid,
                    action_t_stamp,
                    action_host,
                    action_source,
                    action_country
                FROM
                    tmp_conversions
                GROUP BY
                    conversionid, campaignid
                ORDER BY
                    conversionlogid';
    debug('- phpAds_countConversions() - Insertion of temporary table results into conversion log: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Insertion of temporary table results into conversion log: Ended');

	/* Flag the latest conversion for each action, so that we dont get more than
	 * one conversion for each tracker hit
	 */
	
	// Start by getting all the conversionlogids with the latest timestamp for each conversionid
	$sql = 'CREATE TEMPORARY TABLE tmp_latest
                SELECT
                    conversionlogid, t_stamp
                FROM
                    '.$conf['tbl_conversionlog'].'
                WHERE
                    t_stamp >=    '.$begin_timestamp.'
                    AND t_stamp < '.$end_timestamp.'
                GROUP BY
                    conversionid
                HAVING
                    t_stamp = max(t_stamp)';
    debug('- phpAds_countConversions() - Get conversionlogid of latest conversions: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Get conversionlogid of latest conversions: Ended');

	// Update cnv_latest field on tbl_conversionlog table for each conversionlogid in the tmp table
	$sql = 'UPDATE
            '.$conf['tbl_conversionlog'].' as c,
                tmp_latest as t
            SET
                c.cnv_latest=1,
                c.t_stamp=c.t_stamp
            WHERE 
                c.conversionlogid=t.conversionlogid';
    debug('- phpAds_countConversions() - Set \'latest conversion\' flag: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Set \'latest conversion\' flag: Ended');

	$sql = 'DROP TEMPORARY TABLE tmp_latest';
    debug('- phpAds_countConversions() - Drop temporary table: Started');
    $res = phpAds_dbQuery($sql)
        or $report.= "Could not perform SQL: ".$sql."\n";
    debug('- phpAds_countConversions() - Drop temporary table: Ended');

    // Now, add up all of the conversions that we just logged and put them into adstats.
    // Process conversions...
    $num_conversions    = 0;
    $total_conversions  = 0;

    $conversion_query = "
                SELECT action_bannerid,
                    action_zoneid,
                    count(*) as conversions
                FROM " . $conf['tbl_conversionlog'] . "
                WHERE t_stamp >= " . $begin_timestamp . "
                    AND t_stamp < " . $end_timestamp . "
                    AND cnv_logstats = 'y'
					AND cnv_latest = 1
                GROUP BY action_bannerid, action_zoneid";
    debug('- phpAds_countConversions() - Adstats conversions query: Started');
    $conversion_result = phpAds_dbQuery($conversion_query)
        or $report.= "Could not perform SQL: ".$conversion_query."\n";
    debug('- phpAds_countConversions() - Adstats conversions query: Ended');

    $x = 1;
    while ($conversion_row = phpAds_dbFetchArray($conversion_result)) {
        $stat_query = " UPDATE " . $conf['tbl_adstats'] . "
                        SET conversions = conversions + " . $conversion_row['conversions'] . "
                        WHERE day = '" . $day . "'" . "
                        AND hour = " . $hour . "
                        AND bannerid = " . $conversion_row['action_bannerid'] . "
                        AND zoneid = " . $conversion_row['action_zoneid'];
        debug('- phpAds_countConversions() - Conversion query update: Started iteration ' . $x);
        $stat_result = phpAds_dbQuery($stat_query)
            or $report.= " Could not perform SQL: ".$stat_query."\n";
        debug('- phpAds_countConversions() - Conversion query update: Ended iteration ' . $x);

        if (phpAds_dbAffectedRows($stat_result) < 1) {
            $stat_query = " INSERT INTO " . $conf['tbl_adstats'] . "
                            SET day = '".$day."'". ",
                            hour = " . $hour . ",
                            bannerid = " . $conversion_row['action_bannerid']. ",
                            zoneid = " . $conversion_row['action_zoneid'] . ",
                            conversions = " . $conversion_row['conversions'];
            debug('- phpAds_countConversions() - Conversion query insert: Started iteration ' . $x);
            $stat_result = phpAds_dbQuery($stat_query)
                or $report.= " Could not perform SQL: ".$stat_query."\n";
            debug('- phpAds_countConversions() - Conversion query insert: Ended iteration ' . $x);
        }
        $num_conversions += $conversion_row['conversions'];
        $x++;
    }

    $conversion_query = "SELECT".
                    " COUNT(*) as conversions".
                    " FROM ".$conf['tbl_conversionlog'].
                    " WHERE t_stamp>=".$begin_timestamp.
                    " AND t_stamp<".$end_timestamp;
    debug('- phpAds_countConversions() - Select conversions: Started');
    $total_conversions = phpAds_dbQuery($conversion_query)
        or $report.= "Could not perform SQL: ".$conversion_query."\n";
    debug('- phpAds_countConversions() - Select conversions: Ended');

    $conversions = phpAds_dbFetchArray($total_conversions);

    // Update the variablevalues table with the conversionid of the conversion associated
    // with the variable, obtaining and then dropping locks on the appropriate tables.
    //
    // BEWARE!!!
    //
    // The tables names specified with the "AS" statement in the lock MUST be the same as
    // the "AS" table names specified in the update!
    //$variables_query = 'LOCK TABLES
    //                         '.$conf['tbl_adconversions'].' as c READ
    //                        ,'.$conf['tbl_variablevalues'].' as v WRITE';
    //debug('- phpAds_countConversions() - Lock tables for update variablevalues with conversionid associated with that variable: Started');
    //$variables_result = phpAds_dbQuery($variables_query)
    //    or $report.= "Could not perform SQL: ".$variables_query."\n";
    //debug('- phpAds_countConversions() - Lock tables for update variablevalues with conversionid associated with that variable: Ended');
                        
    $variables_query = 'UPDATE 
                             '.$conf['tbl_adconversions'].' as c
                            ,'.$conf['tbl_variablevalues'].' as v
                        SET
                            v.conversionid=c.conversionid
                        WHERE
                            v.local_conversionid=c.local_conversionid 
                            AND v.dbserver_ip=c.dbserver_ip 
                            AND c.t_stamp >= \''.$begin_timestamp.'\' 
                            AND c.t_stamp <  \''.$end_timestamp.'\' 
                            AND v.t_stamp >= \''.$begin_timestamp.'\' 
                            AND v.t_stamp <  \''.$end_timestamp.'\'';
    debug('- phpAds_countConversions() - Update variablevalues with conversionid associated with that variable: Started');
    $variables_result = phpAds_dbQuery($variables_query)
        or $report.= "Could not perform SQL: ".$variables_query."\n";
    debug('- phpAds_countConversions() - Update variablevalues with conversionid associated with that variable: Ended');
                        
    //$variables_query = 'UNLOCK TABLES';
    //debug('- phpAds_countConversions() - Unlock tables after update variablevalues with conversionid associated with that variable: Started');
    //$variables_result = phpAds_dbQuery($variables_query)
    //    or $report.= "Could not perform SQL: ".$variables_query."\n";
    //debug('- phpAds_countConversions() - Unlock tables after update variablevalues with conversionid associated with that variable: Ended');

    $report .= "\tLogged ".$num_conversions." conversions out of ".$conversions['conversions']." in ".(time()-$time)." seconds.\n\n";
}






?>